//SOMOIIver10.inc
//February 4, 2014

//constants for indicating the status of the SOMO II module (from the datasheet)
#CONST
    SOMO_STAT_ACK                // 0 - Command Successfully Received
    SOMO_STAT_ERR_BUSY           // 1 - Module is busy
    SOMO_STAT_ERR_SLEEP          // 2 - Module is currently in sleep mode
    SOMO_STAT_ERR_SERIAL         // 3 - Serial received an error
    SOMO_STAT_ERR_CHKSUM         // 4 - Checksum error
    SOMO_STAT_ERR_SCOPE          // 5 - Beyond scope of specified file/folder
    SOMO_STAT_ERR_NOTFOUND       // 6 - Specified file/Folder not found
    SOMO_STAT_MSG_USB_INSRT      // 7 - USB Flash Drive Inserted. Sent every time a USD Flash drive is plugged in.
    SOMO_STAT_MSG_uSD_INSRT      // 8 - micro-SD Card Inserted. Sent every time a micro-SD card is inserted.
    SOMO_STAT_MSG_USB_RMOVD      // 9 - USB Flash Drive Removed. Sent every time a USD Flash drive is unplugged.
    SOMO_STAT_MSG_uSD_RMOVD      //10 - micro-SD Card Removed. Sent every time a micro-SD card is ejected.
    SOMO_STAT_MSG_USB_FIN        //11 - USB Flash Drive source, Finished playing a track
    SOMO_STAT_MSG_uSD_FIN        //12 - uSD card source, Finished playing a track
    SOMO_STAT_MSG_STRTUP         //13 - SOMO-II has just started up, No media detected
    SOMO_STAT_MSG_STRTUP_USB     //14 - SOMO-II has just started up, USB Flash Drive detected
    SOMO_STAT_MSG_STRTUP_uSD     //15 - SOMO-II has just started up, micro-SD Card detected
    SOMO_STAT_MSG_STRTUP_BOTH    //16 - SOMO-II has just started up, Both micro-SD and USB Flash Drive detected
#END

//Define the port to be used here
//#constant SOMO_PORT COM1    /* COM0 or COM1 */


#IF (SOMO_PORT == COM0)
    #constant somoport_Init     $com_Init
    #constant somoport_Count    $com_Count
    #constant somoport_out      $serout
    #constant somoport_in       $serin

#ELSE
    #constant somoport_Init     $com1_Init
    #constant somoport_Count    $com1_Count
    #constant somoport_out      $serout1
    #constant somoport_in       $serin1
#ENDIF

//6 SOMO replies will be buffered
var somoReply1[7], somoReply2[7], somoReply3[7], somoReply4[7], somoReply5[7], somoReply6[7];
var packetPointer[6] := [somoReply1, somoReply2, somoReply3, somoReply4, somoReply5, somoReply6];

//strings for printing the current status of the SOMO II module
var SOMO_stat[17] := ["successful", "busy", "sleep mode", "serial error", "chksum error",
    "beyond scope", "not found", "USB flash drive inserted", "uSD card inserted",
    "USB flash drive removed", "uSD card removed", "USB: finished playing",
    "uSD: finished playing", "start up, no media", "start up, USB", "start up, uSD",
    "start up, USB + uSD"];

//constants for indicating communication errors
#CONST
    SOMO_COMMS_ERR_CMPLT_DATA       //0 - packet received from the SOMO II module is complete, checksum not yet checked
    SOMO_COMMS_ERR_TM_OUT_NO_DATA   //1 - timeout and no data received from the SOMO II module
    SOMO_COMMS_ERR_TM_OUT_INC_DATA  //2 - timeout and incomplete packet received from the SOMO II module
    SOMO_COMMS_ERR_INVALID_CHKSUM   //3 - packet is complete but checksum is invalid
    SOMO_COMMS_ERR_SUCCESS          //4 - packet is complete and checksum is valid
#END

var SOMO_lastPlayedTrk := 0xFFFF;   //
var SOMO_timeOutLimit := 10000; //10 - second timeout limit
var SOMO_combuf[7];

func getReply(var *reply, var timeLimit)
    var combuf[10];      //RX buffer for up to 20 bytes
    var _rcvdMsgSOMO[7]; //array for storing replies from the SOMO II module
    var read, n ;
    var startTime;
    var private readCount := 0;
    var private chkSumCalc, chkSumRcvd;
    var _SOMOcommsError;

    sys_SetTimer(TIMER1, 0xFFFF);   //start timer count down now
    startTime := sys_GetTimer(TIMER1);
    somoport_Init(combuf, 20, 0x7E);
    readCount := 0;

    if(!timeLimit)
        //wait for 7 bytes or for the timeout limit
        while ((readCount <= 6) && (startTime - sys_GetTimer(TIMER1) < SOMO_timeOutLimit))
            readCount := somoport_Count();
        wend

    else if(timeLimit)
        //wait for 7 bytes or for the timeout limit
        while ((readCount <= 6) && (startTime - sys_GetTimer(TIMER1) < timeLimit))
            readCount := somoport_Count();
        wend
    endif

    if(readCount == 0x0007)                 // if message is complete
        _SOMOcommsError := SOMO_COMMS_ERR_CMPLT_DATA;
        for( n := 0; n <= 6; n++)
            _rcvdMsgSOMO[n] := somoport_in();     // copy the 7 bytes we received to _rcvdMsgSOMO
        next

    else if(readCount > 0 && readCount < 0x0007) //if message is incomplete
        _SOMOcommsError := SOMO_COMMS_ERR_TM_OUT_INC_DATA;
        for( n := 0; n <= readCount; n++)
            _rcvdMsgSOMO[n] := somoport_in();     // copy the incomplete bytes we received to _rcvdMsgSOMO
        next
    else if(!readCount)
        _SOMOcommsError := SOMO_COMMS_ERR_TM_OUT_NO_DATA;
    endif

    if(_SOMOcommsError == SOMO_COMMS_ERR_CMPLT_DATA)
    //calculated checksum  :=   0xFFFF - (  CMD      + Feedback   +    Para1   +   Para2    ) + 1;
        chkSumCalc         :=   0xFFFF - (_rcvdMsgSOMO[0] + _rcvdMsgSOMO[1] + _rcvdMsgSOMO[2] + _rcvdMsgSOMO[3] ) + 1;
        chkSumRcvd         :=   _rcvdMsgSOMO[4] << 8 | _rcvdMsgSOMO[5];

        if(chkSumCalc == chkSumRcvd)
            _SOMOcommsError := SOMO_COMMS_ERR_SUCCESS;
        else
            _SOMOcommsError := SOMO_COMMS_ERR_INVALID_CHKSUM;
        endif
    endif

    if(_SOMOcommsError != SOMO_COMMS_ERR_SUCCESS && !timeLimit)
        commsErrorHandler(_SOMOcommsError, _rcvdMsgSOMO);

    //else if(_SOMOcommsError != SOMO_COMMS_ERR_SUCCESS && timeLimit)
        //do nothing

    else if(_SOMOcommsError == SOMO_COMMS_ERR_SUCCESS)//if successful, copy _rcvdMsgSOMO to "reply"
        for( n := 0; n <= 6; n++)
            reply[n] := _rcvdMsgSOMO[n];
        next
    endif

    somoport_Init(SOMO_combuf, 14, 0x7E);    //wait for the next message, whatever it will be. Store to universal buffer SOMO_combuf[7]

    return _SOMOcommsError;
endfunc

func commsErrorHandler(var commsError, var *reply)
    var n;
    if(commsError == SOMO_COMMS_ERR_TM_OUT_NO_DATA)
        print("Error: timeout - no data.\n");
    else if(commsError == SOMO_COMMS_ERR_TM_OUT_INC_DATA)
        print("Error: timeout - incomplete data.\n");
        print("Received message: ");
        for( n := 0; n < getReply.readCount; n++)
            print([HEX]reply[n]," ");
        next
        print("\n");
    else if(commsError == SOMO_COMMS_ERR_INVALID_CHKSUM)
        print("Error: invalid checksum.\n");
        print("received checksum: ", [HEX] getReply.chkSumRcvd, "\n");
        print("calculated checksum: ", [HEX] getReply.chkSumCalc, "\n");
    endif

    print("Check the SOMO II module\nand reset the display.");
    repeat
    forever
endfunc

func evaluateReply(var *reply)
    var statusIx;

    if(reply[0] == 0x0041 && reply[3] == 0x0000 )
        statusIx := SOMO_STAT_ACK;

    else if(reply[0] == 0x0040)
        if(reply[3] == 0x0001)
            statusIx := SOMO_STAT_ERR_BUSY;
        else if(reply[3] == 0x0002)
            statusIx := SOMO_STAT_ERR_SLEEP;
        else if(reply[3] == 0x0003)
            statusIx := SOMO_STAT_ERR_SERIAL;
        else if(reply[3] == 0x0004)
            statusIx := SOMO_STAT_ERR_CHKSUM;
        else if(reply[3] == 0x0005)
            statusIx := SOMO_STAT_ERR_SCOPE;
        else if(reply[3] == 0x0006)
            statusIx := SOMO_STAT_ERR_NOTFOUND;
        endif

    else if(reply[0] == 0x003A && reply[3] == 0x0001 )
        statusIx := SOMO_STAT_MSG_USB_INSRT;
    else if(reply[0] == 0x003A && reply[3] == 0x0002 )
        statusIx := SOMO_STAT_MSG_uSD_INSRT;
    else if(reply[0] == 0x003B && reply[3] == 0x0001 )
        statusIx := SOMO_STAT_MSG_USB_RMOVD;
    else if(reply[0] == 0x003B && reply[3] == 0x0002 )
        statusIx := SOMO_STAT_MSG_uSD_RMOVD;
    else if(reply[0] == 0x003C)
        SOMO_lastPlayedTrk := reply[3];
        statusIx := SOMO_STAT_MSG_USB_FIN;
    else if(reply[0] == 0x003D)
        SOMO_lastPlayedTrk := reply[3];
        statusIx := SOMO_STAT_MSG_uSD_FIN;

    else if(reply[0] == 0x003F)
        if(reply[3] == 0x0000)
            statusIx := SOMO_STAT_MSG_STRTUP;
        else if(reply[3] == 0x0001)
            statusIx := SOMO_STAT_MSG_STRTUP_USB;
        else if(reply[3] == 0x0002)
            statusIx := SOMO_STAT_MSG_STRTUP_uSD;
        else if(reply[3] == 0x0003)
            statusIx := SOMO_STAT_MSG_STRTUP_BOTH;
        endif
    endif

    return statusIx;
endfunc

func initializeSomo(var attemptMAX)
    var _startUpFlag := 0, _attempt := 0 ;
    var _somoStatusIx;
    var _replySOMO[7];
    while(!_startUpFlag && _attempt < attemptMAX)
        gfx_Cls();
        print("Initializing the SOMO II module...",_attempt,"\n");
        resetSOMO();
        getReply(_replySOMO, 0);
        _somoStatusIx := evaluateReply(_replySOMO);
        if(_somoStatusIx == SOMO_STAT_MSG_STRTUP_uSD || _somoStatusIx == SOMO_STAT_MSG_STRTUP_BOTH)
            _startUpFlag := 1;
        endif
        _attempt++;
        pause(500); //pause for 500 ms before sending the next reset command
    wend
    if(_attempt >= attemptMAX)
        print("Last status: ", [STR]SOMO_stat[_somoStatusIx],"\n");
        pause(2000);
    endif
    return _startUpFlag;
endfunc

func somoBufferEvents()
    var i, chkSumCalc, chkSumRcvd;
    var private packetCounter := -1;
    var _rcvdMsgSOMO[7];
    var temp;
    var private bufferFullFlag := 0;

    if(somoport_Count() >= 7 && !bufferFullFlag)
        //gfx_MoveTo(120, 0);
        //print("com_count: ", com_Count(), "\n");
        for(i := 0; i <=6; i++)
            _rcvdMsgSOMO[i] := somoport_in();
        next
        //calculated checksum  :=   0xFFFF - (  CMD      + Feedback   +    Para1   +   Para2    ) + 1;
        chkSumCalc :=   0xFFFF - (_rcvdMsgSOMO[0] + _rcvdMsgSOMO[1] + _rcvdMsgSOMO[2] + _rcvdMsgSOMO[3] ) + 1;
        chkSumRcvd :=   _rcvdMsgSOMO[4] << 8 | _rcvdMsgSOMO[5];

        if(chkSumCalc == chkSumRcvd)
            packetCounter++;
            temp := packetPointer[packetCounter];
            for(i := 0; i <=6; i++)
                temp[i] := _rcvdMsgSOMO[i];
            next
            /*
            for(counter := 0; counter <=6; counter++)
                print("combuf",counter,": ",[HEX]packetPointer[packetCounter][i],"\n");
            next
            gfx_MoveTo(0, 0);*/
            somoport_Init(SOMO_combuf, 14, 0x7E);//wait for the next message, whatever it will be. Store to universal buffer SOMO_combuf[7]
            if(packetCounter >= 5)
                packetCounter := 5; //buffer is full, maintain counter at the maximum limit
                bufferFullFlag := 1;
            endif
        endif
    endif

    if(somoport_Count() >= 7 && bufferFullFlag)  somoport_Init(SOMO_combuf, 14, 0x7E);

endfunc

func somoDequeue(var *reply)
    var i, j, temp, temp1, returnValue := 0;
    if(!somoBufferEvents.packetCounter)
        temp := packetPointer[0];
        for(i := 0; i <= 6; i++)//transfer the contents of somoReply1 to the 'working' array
             reply[i] := temp[i];
        next
        returnValue := 1;
        somoBufferEvents.packetCounter := -1; //buffer is empty

    else if(somoBufferEvents.packetCounter != -1 && somoBufferEvents.packetCounter != 0)
        temp := packetPointer[0];
        for(i := 0; i <= 6; i++)//transfer the contents of somoReply1 to the 'working' array
             reply[i] := temp[i];
        next
        //subtract 1 from the packetCounter (but check if it is equal to zero first)
        for(i := 0; i < somoBufferEvents.packetCounter; i++)
            temp := packetPointer[i];
            temp1 := packetPointer[i + 1];
            for(j := 0; j <= 6; j++)
                temp[j] := temp1[j];
            next
        next
        returnValue := 1;
        somoBufferEvents.packetCounter--;
        somoBufferEvents.bufferFullFlag := 0;
    endif
    return returnValue;
endfunc

func sendPacket(var * packetArray, var size)
    var _ix;
    for (_ix := 0; _ix < size; _ix++)
        somoport_out(packetArray[_ix]);
    next
endfunc

//Play Song
func playSong()
    var _Data[8] := [0x7E, 0x0D, 0x00, 0x00, 0x00, 0xFF, 0xF3, 0xEF];
    sendPacket(_Data, 8);
    //com_Init(SOMO_combuf, 14, 0x7E);    //wait for the next message, whatever it will be. Store to universal buffer SOMO_combuf[7]
endfunc

//Stop Song
func stopSong()
    var _Data[8] := [0x7E, 0x16, 0x00, 0x00, 0x00, 0xFF, 0xEA, 0xEF];
    sendPacket(_Data, 8);
endfunc

//Pause Song
func pauseSong()
    var _Data[8] := [0x7E, 0x0E, 0x00, 0x00, 0x00, 0xFF, 0xF2, 0xEF];
    sendPacket(_Data, 8);
    //com_Init(SOMO_combuf, 14, 0x7E);    //wait for the next message, whatever it will be. Store to universal buffer SOMO_combuf[7]
endfunc

//Next Song
func nextSong()
    var _Data[8] := [0x7E, 0x01, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xEF];
    sendPacket(_Data, 8);
    //com_Init(SOMO_combuf, 14, 0x7E);    //wait for the next message, whatever it will be. Store to universal buffer SOMO_combuf[7]
endfunc

//Previous Song
func previousSong()
    var _Data[8] := [0x7E, 0x02, 0x00, 0x00, 0x00, 0xFF, 0xFE, 0xEF];
    sendPacket(_Data, 8);
    //com_Init(SOMO_combuf, 14, 0x7E);    //wait for the next message, whatever it will be. Store to universal buffer SOMO_combuf[7]
endfunc

//Volume#
func volumeSet(var volume)
    var _checksumLow, _checksumHigh;
    _checksumLow := 255 - (6 + volume) + 1;
    _checksumHigh := 255;

    var _Data[8] := [0x7E, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0xEF];
    _Data[4] := volume;
    _Data[5] := _checksumHigh;
    _Data[6] := _checksumLow;
    sendPacket(_Data, 8);
endfunc

//Select uSD Source
func selectuSD()
    var _Data[8] := [0x7E, 0x09, 0x00, 0x00, 0x02, 0xFF, 0xF5, 0xEF];
    sendPacket(_Data, 8);
endfunc

//Query tracks uSD card
/*
func selectuSD()
    var _Data[8] := [0x7E, 0x09, 0x00, 0x00, 0x02, 0xFF, 0xF5, 0xEF];
    sendPacket(_Data, 8);
endfunc  */

//Query currently playing track uSD
func queryCurrentTrackuSD(var timeLimit)
    var _success := 0; //0 - fail, 1 - success
    var i;
    var private _rcvdMsgSOMO[7]; //array for storing replies from the SOMO II module
    var _Data[8] := [0x7E, 0x4C, 0x00, 0x00, 0x00, 0xFF, 0xB4, 0xEF];
    sendPacket(_Data, 8);
    if(getReply(_rcvdMsgSOMO, timeLimit) == SOMO_COMMS_ERR_SUCCESS)
        if(_rcvdMsgSOMO[0] == 0x004C )    _success := 1;
    else
        _success := 0;
    endif

    return _success;
endfunc

//Query the number of files present on the uSD card
func queryTracksuSD(var timeLimit)
    var _success := 0; //0 - fail, 1 - success
    var i;
    var private _rcvdMsgSOMO[7]; //array for storing replies from the SOMO II module
    var _Data[8] := [0x7E, 0x48, 0x00, 0x00, 0x00, 0xFF, 0xB8, 0xEF];
    sendPacket(_Data, 8);
    if(getReply(_rcvdMsgSOMO, timeLimit) == SOMO_COMMS_ERR_SUCCESS)
        if(_rcvdMsgSOMO[0] == 0x0048 )    _success := 1;
    else
        _success := 0;
    endif

    return _success;
endfunc

//Specify Track#
func specifyTrack(var track)
    var _checksumLow, _checksumHigh;
    _checksumLow := 255 - (0x03 + track) + 1;
    _checksumHigh := 255;

    var _Data[8] := [0x7E, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0xEF];
    _Data[4] := track;
    _Data[5] := _checksumHigh;
    _Data[6] := _checksumLow;
    sendPacket(_Data, 8);
endfunc

//Specify EQ
func specifyEQ(var eq)
    var _checksumLow, _checksumHigh;
    _checksumLow := 255 - (0x07 + eq) + 1;
    _checksumHigh := 255;

    var _Data[8] := [0x7E, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0xEF];
    _Data[4] := eq;
    _Data[5] := _checksumHigh;
    _Data[6] := _checksumLow;
    sendPacket(_Data, 8);
endfunc


//Query current EQ
/*
func selectuSD()
    var _Data[8] := [0x7E, 0x09, 0x00, 0x00, 0x02, 0xFF, 0xF5, 0xEF];
    sendPacket(_Data, 8);
endfunc  */

//Put SOMO-II to Sleep
func sleepSOMO()
    var _Data[8] := [0x7E, 0x0A, 0x00, 0x00, 0x00, 0xFF, 0xF6, 0xEF];
    sendPacket(_Data, 8);
endfunc

//Reset SOMO-II
func resetSOMO()
    var _Data[8] := [0x7E, 0x0C, 0x00, 0x00, 0x00, 0xFF, 0xF4, 0xEF];
    sendPacket(_Data, 8);
endfunc

//single play mode
/*This will disable CONTINUOUS, RANDOM TRACK or REPEAT CURRENT modes if previously enabled, which is how the module starts up by default. This will allow one song to play and then stop. Start track with PLAY.*/
func singlePlay()
    var _Data[8] := [0x7E, 0x19, 0x00, 0x00, 0x01, 0xFF, 0xE6, 0xEF];
    sendPacket(_Data, 8);
endfunc

//continuous play mode
/*This will enable continuous mode (disable RANDOM TRACK and REPEAT CURRENT if previously enabled),
which will play all songs on the memory card, one after the other. Start track with PLAY.*/
func continuousPlay()
    var _Data[8] := [0x7E, 0x11, 0x00, 0x00, 0x01, 0xFF, 0xEE, 0xEF];
    sendPacket(_Data, 8);
endfunc

//random play mode
/*This will enable Random Mode (disable CONTINUOUS and REPEAT CURRENT if previously enabled), which plays random tracks one after the other, continuously. Start track with PLAY.*/
func randomPlay()
    var _Data[8] := [0x7E, 0x18, 0x00, 0x00, 0x00, 0xFF, 0xE8, 0xEF];
    sendPacket(_Data, 8);
endfunc

//repeat current play mode
/*This will enable the repeat play mode (disable CONTINUOUS and RANDOM TRACK if previously enabled), which repeats the currently playing track, so it will play over and over continuously.*/
func repeatCurrent()
    var _Data[8] := [0x7E, 0x19, 0x00, 0x00, 0x00, 0xFF, 0xE7, 0xEF];
    sendPacket(_Data, 8);
endfunc

func setPlayMode(var mode)
    if(!mode)
        singlePlay();
    else if(mode == 1)
        continuousPlay();
    else if(mode == 2)
        randomPlay();
    else if(mode == 3)
        repeatCurrent();
    endif
endfunc




